__copyright__ = "Copyright 2014 Davide Olianas (ubuntupk@gmail.com)."

__license__ = """This file is part of poliBeePsync.
poliBeePsync is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

poliBeePsync is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with poliBeePsync. If not, see <http://www.gnu.org/licenses/>.
"""

from polibeepsync import filesettings
from requests import ConnectionError, Timeout
from appdirs import user_config_dir, user_data_dir
import os
import pickle
import sys
from polibeepsync.common import User, InvalidLoginError
from PySide.QtCore import (QThread, QObject, Signal, QAbstractTableModel,
                           QModelIndex, Qt, Slot, QTimer, QLocale, QPoint)
from PySide.QtGui import (QApplication, QWidget, QTextCursor,
                          QMenu, QAction, QFileDialog,
                          QVBoxLayout, QLabel, QSystemTrayIcon,
                          qApp, QDialog)

from polibeepsync.ui_resizable import Ui_Form
import re
import logging
import sys

LEVELS = {'debug': logging.DEBUG,
          'info': logging.INFO,
          'warning': logging.WARNING,
          'error': logging.ERROR,
          'critical': logging.CRITICAL,
}

if len(sys.argv) > 1:
    level_name = sys.argv[1]
    level = LEVELS.get(level_name, logging.NOTSET)
    logging.basicConfig(level=level)


def read(*names, **kwargs):
    with open(
            os.path.join(os.path.dirname(__file__), *names),
            encoding=kwargs.get("encoding", "utf8")
    ) as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


__version__ = find_version("__init__.py")


class MySignal(QObject):
    sig = Signal(str)


class CoursesSignal(QObject):
    sig = Signal(list)


class DownloadThread(QThread):
    def __init__(self, user, topdir, parent=None):
        QThread.__init__(self, parent)
        self.exiting = False
        self.course_finished = MySignal()
        self.signal_error = MySignal()
        self.user = user
        self.topdir = topdir
        self.dumpuser = MySignal()

    def run(self):
        while self.exiting == False:
            for course in self.user.available_courses:
                subdir = course.save_folder_name
                if course.sync == True:
                    try:
                        self.user.update_course_files(course)
                        outdir = os.path.join(self.topdir, subdir)
                        os.makedirs(outdir, exist_ok=True)
                        rootdir = self.user.find_files_and_folders(
                            course.documents_url, 'rootfolder')
                        self.user.save_files(rootdir, outdir)
                        self.dumpuser.sig.emit('User object changed')
                        text = "Synced files for {}".format(course.name)
                        self.course_finished.sig.emit(text)
                    except Exception as err:
                        self.signal_error.sig.emit(str(err))
                        logging.critical(str(err))
            self.exiting = True


class LoginThread(QThread):
    def __init__(self, user, parent=None):
        QThread.__init__(self, parent)
        self.exiting = False
        self.signal_ok = MySignal()
        self.signal_error = MySignal()
        self.user = user

    def run(self):
        while self.exiting == False:
            try:
                self.user.logout()
                self.user.login()
                if self.user.logged == True:
                    self.exiting = True
                    self.signal_ok.sig.emit('Successful login.')
            except IndexError:
                logging.critical(str(err))
                self.exiting = True
                self.signal_error.sig.emit('Already logged-in.')
            except InvalidLoginError:
                self.user.logout()
                self.exiting = True
                self.signal_error.sig.emit('Login failed.')
                logging.critical(str(err))
            except ConnectionError as err:
                self.user.logout()
                self.exiting = True
                self.signal_error.sig.emit('I can\'t connect to the server.'
                                           ' Is the Internet connection working?')
                logging.critical(str(err))
            except Timeout as err:
                self.user.logout()
                self.exiting = True
                self.signal_error.sig.emit("The timeout time has been reached."
                                           " Is the Internet connection working?")
                logging.critical(str(err))

            except Exception as err:
                self.user.logout()
                self.exiting = True
                self.signal_error.sig.emit("An error occurred.")
                logging.critical(str(err))


class RefreshCoursesThread(QThread):
    def __init__(self, user, parent=None):
        QThread.__init__(self, parent)
        self.exiting = False
        self.refreshed = MySignal()
        self.dumpuser = MySignal()
        self.newcourses = CoursesSignal()
        self.removable = CoursesSignal()
        self.user = user

    def run(self):
        while self.exiting == False:
            most_recent = self.user.get_online_courses()
            last = self.user.available_courses
            new = most_recent - last
            removable = last - most_recent
            if len(removable) > 0:
                self.refreshed.sig.emit('The following courses have'
                                        ' been removed because they '
                                        'aren\'t available online: {}'.format(
                    removable))
            if len(new) > 0:
                for course in new:
                    course.save_folder_name = course.simplify_name(course.name)
                    self.refreshed.sig.emit('A new course '
                                            'was found: {}'.format(course))
            if len(new) == 0:
                self.refreshed.sig.emit('No new courses found.')
            self.user.sync_available_courses(most_recent)
            self.dumpuser.sig.emit('User object changed')
            self.newcourses.sig.emit(new)
            self.removable.sig.emit(removable)
            self.exiting = True
            # nel main thread chaimare dumpUser()
            # e emettere segnale che passa new e removable

            # for course in new:
            # self.courses_model.insertRows(0, 1, course)
            #for course in removable:
            #    index = self.courses_model.courses.index(course)
            #    self.courses_model.removeRows(index, 1)


class CoursesListModel(QAbstractTableModel):
    def __init__(self, courses):
        QAbstractTableModel.__init__(self)
        # il mio è un mapping, mentre con tableview viene più comodo avere indici
        self.courses = list(courses)

    def rowCount(self, parent=QModelIndex()):
        return len(self.courses)

    def columnCount(self, parent=QModelIndex()):
        return 3

    def insertRows(self, position, rows, newcourse, parent=QModelIndex()):
        self.beginInsertRows(parent, position, position + rows - 1)
        for row in range(rows):
            self.courses.insert(position, newcourse)
        self.endInsertRows()
        return True

    def removeRows(self, position, rows, parent=QModelIndex()):
        self.beginRemoveRows(parent, position, position + rows - 1)
        for row in range(rows):
            del self.courses[position]
        self.endRemoveRows()
        return True

    def flags(self, index):
        if index.column() == 2:
            flags = Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable
            return flags
        elif index.column() == 1:
            flags = Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable \
                    | Qt.ItemIsUserCheckable
            return flags
        else:
            return Qt.ItemIsEnabled

    def setData(self, index, value, role=Qt.EditRole):
        if role == Qt.EditRole:
            if index.column() == 2:
                other_names = [elem.save_folder_name for elem in self.courses]
                if value not in other_names:
                    self.courses[index.row()].save_folder_name = value
                    self.dataChanged.emit(index, index)
                return True
            elif index.column() == 1:
                self.courses[index.row()].sync = value
                self.dataChanged.emit(index, index)
                return True
        return False


    def data(self, index, role=Qt.DisplayRole):
        if role == Qt.DisplayRole:
            if index.column() == 0:
                return self.courses[index.row()].name
            if index.column() == 1:
                return self.courses[index.row()].sync
            if index.column() == 2:
                return self.courses[index.row()].save_folder_name
        elif role == Qt.CheckStateRole:
            return None

    def headerData(self, col, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            if col == 0:
                return "Name"
            elif col == 1:
                return "Sync"
            elif col == 2:
                return "Save as"


class MainWindow(QWidget, Ui_Form):
    def __init__(self, parent=None):
        super(MainWindow, self).__init__(parent)
        self.appname = "poliBeePsync"
        self.settings_fname = 'pbs-settings.ini'
        self.data_fname = 'pbs.data'
        self.setupUi(self)
        self.w = QWidget()
        self.createTray()
        self.about.clicked.connect(self.about_box)
        self.timer = QTimer(self)

        self.load_settings()
        self.load_data()

        self.timer.timeout.connect(self.syncfiles)
        self.timer.start(1000 * 60 * int(self.settings['UpdateEvery']))

        self.loginthread = LoginThread(self.user)
        self.loginthread.signal_error.sig.connect(self.myStream_message)
        self.loginthread.signal_error.sig.connect(self.loginstatus)
        self.loginthread.signal_ok.sig.connect(self.myStream_message)
        self.loginthread.signal_ok.sig.connect(self.loginstatus)

        self.refreshcoursesthread = RefreshCoursesThread(self.user)
        self.refreshcoursesthread.dumpuser.sig.connect(self.dumpUser)
        self.refreshcoursesthread.newcourses.sig.connect(self.addtocoursesview)
        self.refreshcoursesthread.newcourses.sig.connect(self.syncnewcourses)
        self.refreshcoursesthread.refreshed.sig.connect(self.myStream_message)
        self.refreshcoursesthread.removable.sig.connect(self.rmfromcoursesview)

        self.downloadthread = DownloadThread(self.user,
                                             self.settings['RootFolder'])
        self.downloadthread.dumpuser.sig.connect(self.dumpUser)
        self.downloadthread.course_finished.sig.connect(self.myStream_message)
        self.downloadthread.signal_error.sig.connect(self.myStream_message)

        self.userCode.setText(str(self.user.username))
        self.userCode.textEdited.connect(self.setusercode)
        self.password.setText(self.user.password)
        self.password.textEdited.connect(self.setpassword)
        self.trylogin.clicked.connect(self.testlogin)

        self.courses_model = CoursesListModel(self.user.available_courses)
        self.coursesView.setModel(self.courses_model)
        self.refreshCourses.clicked.connect(self.refreshcourses)
        self.courses_model.dataChanged.connect(self.dumpUser)
        self.syncNow.clicked.connect(self.syncfiles)

        if self.settings['SyncNewCourses'] == str(True):
            self.sync_new = Qt.Checked
        else:
            self.sync_new = Qt.Unchecked

        # if self.settings['NotifyNewCourses'] == str(True):
        # self.notify_new = Qt.Checked
        #else:
        #    self.notify_new = Qt.Unchecked

        self.rootfolder.setText(self.settings['RootFolder'])
        self.rootfolder.textChanged.connect(self.rootfolderslot)

        #self.notifyNewCourses.setCheckState(self.notify_new)

        #self.notifyNewCourses.stateChanged.connect(self.notifynew)


        self.addSyncNewCourses.setCheckState(self.sync_new)
        self.addSyncNewCourses.stateChanged.connect(self.syncnewslot)

        self.timerMinutes.setValue(int(self.settings['UpdateEvery']))
        self.timerMinutes.valueChanged.connect(self.updateminuteslot)
        #self.timerMinutes.valueChanged.connect(STUFF)

        self.changeRootFolder.clicked.connect(self.chooserootdir)

    def syncnewcourses(self, newlist):
        if self.settings['SyncNewCourses'] == 'True':
            for elem in newlist:
                elem.sync = True


    def load_settings(self):
        for path in [user_config_dir(self.appname),
                     user_data_dir(self.appname)]:
            try:
                os.makedirs(path, exist_ok=True)
            except OSError as err:
                logging.critical(str(err))
                if not os.path.isdir(path):
                    self.myStream_message(str(err))
        self.settings_path = os.path.join(user_config_dir(self.appname),
                                          self.settings_fname)
        defaults = {
            'UpdateEvery': '60',
            'RootFolder': os.path.join(os.path.expanduser('~'), self.appname),
            'SyncNewCourses': 'False'
        }
        self.settings = filesettings.settingsFromFile(self.settings_path,
                                                      defaults)

    def load_data(self):
        try:
            with open(os.path.join(user_data_dir(self.appname),
                                   self.data_fname), 'rb') as f:
                self.user = pickle.load(f)
                self.myStream_message("Data has been loaded successfully.")
        except FileNotFoundError as err:
            logging.critical(str(err))
            self.user = User('', '')
            complete_message = str(err) + " ".join([
                "\nThis error means that no data can be found in the predefined",
                "directory. Ignore this if you're using poliBeePsync for "
                "the first time."])
            self.myStream_message(complete_message)
        except Exception as err:
            logging.critical(str(err))
            self.user = User('', '')
            self.myStream_message(str(err))

    def loginstatus(self, status):
        self.login_attempt.setText(status)

    # @Slot(int)
    # def notifynew(self, state):
    #    if state == 2:
    #        self.settings['NotifyNewCourses'] = 'True'
    #    else:
    #        self.settings['NotifyNewCourses'] = 'False'
    #    filesettings.settingsToFile(self.settings, self.settings_path)

    @Slot(int)
    def syncnewslot(self, state):
        if state == 2:
            self.settings['SyncNewCourses'] = 'True'
        else:
            self.settings['SyncNewCourses'] = 'False'
        filesettings.settingsToFile(self.settings, self.settings_path)

    @Slot(int)
    def updateminuteslot(self, minutes):
        self.settings['UpdateEvery'] = str(minutes)
        filesettings.settingsToFile(self.settings, self.settings_path)
        self.timer.start(1000 * 60 * int(self.settings['UpdateEvery']))

    @Slot(str)
    def rootfolderslot(self, path):
        self.settings['RootFolder'] = path
        filesettings.settingsToFile(self.settings, self.settings_path)


    def chooserootdir(self):
        currentdir = self.settings['RootFolder']
        flags = QFileDialog.DontResolveSymlinks | QFileDialog.ShowDirsOnly
        newroot = QFileDialog.getExistingDirectory(None,
                                                   "Open Directory",
                                                   currentdir, flags)
        if newroot != "" and str(newroot) != currentdir:
            self.settings['RootFolder'] = str(newroot)
            filesettings.settingsToFile(self.settings, self.settings_path)
            self.rootfolder.setText(newroot)
            # we delete the already present downloadthread and recreate it
            # because otherwise it uses the old download folder. I don't know
            # if there's a cleaner approach
            del self.downloadthread
            self.downloadthread = DownloadThread(self.user,
                                                 self.settings['RootFolder'])
            self.downloadthread.dumpuser.sig.connect(self.dumpUser)
            self.downloadthread.course_finished.sig.connect(
                self.myStream_message)
            self.downloadthread.signal_error.sig.connect(self.myStream_message)


    def setusercode(self):
        newcode = self.userCode.text()
        self.user.username = newcode
        try:
            self.dumpUser()
            if len(newcode) == 8:
                self.myStream_message("User code changed to {}."
                                      .format(newcode))
        except Exception as err:
            self.myStream_message(str(err))
            logging.critical(str(err))


    def setpassword(self):
        newpass = self.password.text()
        self.user.password = newpass
        try:
            self.dumpUser()
            self.myStream_message("Password changed.")
        except Exception as err:
            self.myStream_message(str(err))
            logging.critical(str(err))

    def testlogin(self):
        if not self.loginthread.isRunning():
            self.loginthread.exiting = False
            self.loginthread.start()
            self.login_attempt.setStyleSheet("color: rgba(0, 0, 0, 255);")
            self.login_attempt.setText("Logging in, please wait.")

    def addtocoursesview(self, addlist):
        for elem in addlist:
            self.courses_model.insertRows(0, 1, elem)

    def rmfromcoursesview(self, removelist):
        for elem in removelist:
            index = self.courses_model.courses.index(elem)
            self.courses_model.removeRows(index, 1)


    def dumpUser(self):
        # we don't use the message...
        with open(os.path.join(user_data_dir(self.appname),
                               self.data_fname), 'wb') as f:
            pickle.dump(self.user, f)

    def refreshcourses(self):
        if not self.loginthread.isRunning():
            self.loginthread.exiting = False
            self.loginthread.signal_ok.sig.connect(self.do_refreshcourses)
            self.loginthread.start()

    def do_refreshcourses(self):
        self.loginthread.signal_ok.sig.disconnect(self.do_refreshcourses)
        if not self.refreshcoursesthread.isRunning():
            self.refreshcoursesthread.exiting = False
            self.refreshcoursesthread.start()

    @Slot()
    def syncfiles(self):
        self.refreshcoursesthread.finished.connect(self.do_syncfiles)
        self.refreshcourses()

    def do_syncfiles(self):
        self.refreshcoursesthread.finished.disconnect(self.do_syncfiles)
        if not self.downloadthread.isRunning():
            self.downloadthread.exiting = False
            self.downloadthread.start()


    @Slot(str)
    def myStream_message(self, message):
        self.status.moveCursor(QTextCursor.End)
        self.status.insertPlainText(message + "\n\n")

    def createTray(self):
        restoreAction = QAction("&Restore", self, triggered=self.showNormal)
        quitAction = QAction("&Quit", self, triggered=qApp.quit)
        trayIconMenu = QMenu()
        trayIconMenu.addAction(restoreAction)
        trayIconMenu.addAction(quitAction)
        trayIcon = QSystemTrayIcon(self.icon, self.w)
        trayIcon.setContextMenu(trayIconMenu)
        trayIcon.show()

    def closeEvent(self, event):
        self.hide()
        event.ignore()

    def about_box(self):
        Dialog = QDialog()
        Dialog.setObjectName("Dialog")
        Dialog.resize(379, 161)
        Dialog.setWindowIcon(self.icon)
        verticalLayout = QVBoxLayout(Dialog)
        verticalLayout.setObjectName("verticalLayout")
        label = QLabel(Dialog)
        label.setTextFormat(Qt.RichText)
        label.setOpenExternalLinks(True)
        label.setLocale(QLocale(QLocale.English, QLocale.UnitedStates))
        label.setScaledContents(True)
        label.setWordWrap(True)
        label.setObjectName("label")
        verticalLayout.addWidget(label)
        text = "<html><head/><body><p>poliBeePsync version {}.</p><p>poliBeePsync is a program written by Davide Olianas, released under GNU GPLv3+.</p><p><br/></p><p>More information is available on the <a href=\"http://www.davideolianas.com/polibeepsync\"><span style=\" text-decoration: underline; color:#0000ff;\">official website</span></a>.</p></body></html>".format(
            __version__)
        Dialog.setWindowTitle(
            QApplication.translate("Dialog", "About poliBeePsync", None,
                                   QApplication.UnicodeUTF8))
        label.setText(QApplication.translate("Dialog", text, None,
                                             QApplication.UnicodeUTF8))
        Dialog.exec_()


def main():
    app = QApplication(sys.argv)
    frame = MainWindow()
    frame.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



from behave import *
from os import environ
from polibeepsync.common import User
import requests



@when('I select the courses tab')
def step_impl(context):
    assert False

@then('I get all available courses')
def step_impl(context):
    assert False


@when('I press the button "select all"')
def step_impl(context):
    assert False

@then('All courses should be selected')
def step_impl(context):
    assert False

@when('I press the button "unselect all"')
def step_impl(context):
    assert False

@then('No course should be selected')
def step_impl(context):
    assert False

@when('I\'m presented with the course list')
def step_impl(context):
    assert False

@then('all courses are selected except "BeeP Channel"')
def step_impl(context):
    assert False


from os import environ
from polibeepsync.common import User
from behave import *

def before_scenario(context, scenario):
    """Code to run before every scenario, except login tests"""
    if 'require_login' in scenario.tags:
        usercode = environ["USERCODE"]
        password = environ["PASSWORD"]
        context.user = User(usercode, password)
        context.user.visit()
        context.user.login()



# run this with python buildwin.py bdist_msi
import sys
from cx_Freeze import setup, Executable

base=None
if sys.platform == 'win32':
	base = 'Win32GUI'

executables = [
	Executable('polibeepsync\qtgui.py', base=base)]

setup(name='poliBeePsync',
	version='0.1.1',
	executables=executables)


from polibeepsync.common import Courses, Course, CourseFile, User
import pytest


class TestCourse:
    def test_simplifynamewithsquarebrackets(self):
        course = Course('[2014-15] - OTTICA FISICA E TECNOLOGIE OTTICHE [C.I.] [ INGEGNERIA FISICA ]', 'beep.com')
        assert course.simplify_name(course.name) == "Ottica Fisica E Tecnologie Ottiche C.I."

    def test_simplifysimplename(self):
        course = Course('[2014-15] - SOME STUFF [ A PROFESSOR ]', 'beep.com')
        assert course.simplify_name(course.name) == "Some Stuff"

    def test_difference(self):
        a = CourseFile('a', 'url', '1990')
        b = CourseFile('b', 'url', '111')
        c = CourseFile('c', 'url', '1111')
        e = CourseFile('e', 'url', '19')
        offline_metal = Course('metalli', 'beep.com')
        metal = Course('metalli', 'beep.com')
        offline_metal.append(a, b, c)
        metal.append(a, b, e)
        assert set(metal-offline_metal) == set([e])

    def test_equality(self):
        a = CourseFile('a', 'url', '1990')
        b = CourseFile('b', 'url', '111')
        c = CourseFile('c', 'url', '1111')
        offline_metal = Course('metalli', 'beep.com')
        metal = Course('metalli', 'beep.com')
        offline_metal.append(a, b)
        metal.append(a, b, c)
        assert metal == offline_metal

    def test_inequality(self):
        other = Course('thing', 'beep.com')
        metal = Course('metalli', 'beep.com')
        assert metal != other

    def test_getitem(self):
        a = CourseFile('a', 'url', '1990')
        metal = Course('metalli', 'beep.com')
        metal.append(a)
        assert metal['a'] == a

    def test_getitem_raises_keyerror(self):
        with pytest.raises(KeyError):
            a = CourseFile('a', 'url', '1990')
            metal = Course('metalli', 'beep.com')
            metal.append(a)
            metal['b']

    def test__repr__(self):
        metal = Course('metalli', 'beep.com')
        assert repr(metal) == "Course metalli"

    def test__contains__a_file(self):
        onefile = CourseFile('a', 'url', '1990')
        metal = Course('metalli', 'beep.com')
        metal.append(onefile)
        assert onefile in metal

    def test__doesnt_contain__a_file(self):
        onefile = CourseFile('a', 'url', '1990')
        metal = Course('metalli', 'beep.com')
        assert onefile not in metal

    def test__init__(self):
        metal = Course('metalli', 'beep.com')
        assert hasattr(metal, 'elements')


class TestCourses:
    def test_difference(self):
        offline_metal = Course('metalli', 'beep.com')
        offline_polim = Course('polimeri', 'beep.it')
        metal = Course('metalli', 'beep.com')
        polim = Course('polimeri', 'beep.it')

        offline = Courses()
        offline.append(offline_polim, offline_metal)
        online = Courses()
        online.append(metal, polim)
        for course in online:
            print('online {}'.format(course.name))
        for course in offline:
            print('offline {}'.format(course.name))
        assert online - offline == []

    def test_belonging(self):
        metal = Course('metalli', 'beep.com')
        metalinside = Course('metalli', 'beep.com')
        other = Course('other', 'doesntmatter')
        courses = Courses()
        courses.append(metalinside, other)
        assert metal in courses

    def test_notbelonging(self):
        metal = Course('metalli', 'beep.com')
        courseinside = Course('polimeri', 'beep.com')
        other = Course('other', 'doesntmatter')
        courses = Courses()
        courses.append(courseinside, other)
        assert metal not in courses

    def test_return_hash(self):
        courses = Courses()
        courses.append(Course('metalli', 'beep.com'))
        assert hash(courses) is not None

    def test_hash_not_depending_on_order(self):
        oneorder = Courses()
        otherorder = Courses()
        metal = Course('metalli', 'beep.com')
        polim = Course('polimeri', 'beep.it')
        oneorder.append(metal, polim)
        otherorder.append(polim, metal)
        assert hash(oneorder) == hash(otherorder)

    def test_append(self):
        metal = Course('metalli', 'beep.com')
        other = Course('other', 'doesntmatter')
        oneatatime = Courses()
        # append in two steps
        oneatatime.append(metal)
        oneatatime.append(other)
        twocourses = Courses()
        twocourses.append(other, metal)
        # append in one step
        assert twocourses == oneatatime

    def test_remove(self):
        all = Courses()
        one = Courses()
        metal = Course('metalli', 'beep.com')
        other = Course('other', 'doesntmatter')
        all.append(metal, other)
        one.append(other)
        all.remove(metal)
        assert all == one

    def test__len__(self):
        courses = Courses()
        courses.append(Course('metalli', 'beep.com'),
                       Course('other', 'doesntmatter'))
        assert len(courses) == 2


class TestUser:
    def test_sync_courses_new_course(self):
        guy = User('fakeid', 'fakepwd')
        metal = Course('metalli', 'beep.com')
        other = Course('other', 'doesntmatter')
        online = Courses()
        online.append(metal, other)
        guy.available_courses.append(metal)
        guy.sync_available_courses(online)
        assert guy.available_courses == online

    def test_sync_courses_remove(self):
        guy = User('fakeid', 'fakepwd')
        metal = Course('metalli', 'beep.com')
        other = Course('other', 'doesntmatter')
        online = Courses()
        online.append(metal)
        guy.available_courses.append(metal, other)
        guy.sync_available_courses(online)
        assert guy.available_courses == online



# -*- coding: utf-8 -*-

from PySide.QtGui import (QTableView, QStyledItemDelegate, QStyleOptionButton,
                          QStyle, QApplication, QIcon, QVBoxLayout, QTabWidget,
                          QWidget, QHBoxLayout,
                          QGridLayout, QLabel, QLineEdit, QSpacerItem,
                          QSizePolicy, QPushButton,
                          QSpinBox, QCheckBox, QTextEdit, QDialogButtonBox)
from PySide.QtCore import (QEvent, Qt, QPoint, QRect, QLocale, QSize,
                           QMetaObject)


class CoursesListView(QTableView):
    def __init__(self, parent=None):
        QTableView.__init__(self, parent)
        self.setItemDelegateForColumn(1, CheckBoxDelegate(self))


class CheckBoxDelegate(QStyledItemDelegate):
    """
    A delegate that places a fully functioning QCheckBox in every
    cell of the column to which it's applied
    """

    def __init__(self, parent):
        QStyledItemDelegate.__init__(self, parent)

    def createEditor(self, parent, option, index):
        return None

    def paint(self, painter, option, index):
        checked = bool(index.data())
        check_box_style_option = QStyleOptionButton()

        if (index.flags() & Qt.ItemIsEditable) > 0:
            check_box_style_option.state |= QStyle.State_Enabled
        else:
            check_box_style_option.state |= QStyle.State_ReadOnly

        if checked:
            check_box_style_option.state |= QStyle.State_On
        else:
            check_box_style_option.state |= QStyle.State_Off

        check_box_style_option.rect = self.getCheckBoxRect(option)

        check_box_style_option.state |= QStyle.State_Enabled

        QApplication.style().drawControl(QStyle.CE_CheckBox,
                                         check_box_style_option, painter)

    def editorEvent(self, event, model, option, index):
        if not (index.flags() & Qt.ItemIsEditable) > 0:
            return False

        # Do not change the checkbox-state
        if event.type() == QEvent.MouseButtonPress:
            return False
        if event.type() == QEvent.MouseButtonRelease or event.type() == QEvent.MouseButtonDblClick:
            if event.button() != Qt.LeftButton or not self.getCheckBoxRect(
                    option).contains(event.pos()):
                return False
            if event.type() == QEvent.MouseButtonDblClick:
                return True
        elif event.type() == QEvent.KeyPress:
            if event.key() != Qt.Key_Space and event.key() != Qt.Key_Select:
                return False
            else:
                return False

        # Change the checkbox-state
        self.setModelData(None, model, index)
        return True

    def setModelData(self, editor, model, index):
        newValue = not bool(index.data())
        model.setData(index, newValue, Qt.EditRole)

    def getCheckBoxRect(self, option):
        check_box_style_option = QStyleOptionButton()
        check_box_rect = QApplication.style().subElementRect(
            QStyle.SE_CheckBoxIndicator, check_box_style_option, None)
        check_box_point = QPoint(option.rect.x() +
                                 option.rect.width() / 2 -
                                 check_box_rect.width() / 2,
                                 option.rect.y() +
                                 option.rect.height() / 2 -
                                 check_box_rect.height() / 2)
        return QRect(check_box_point, check_box_rect.size())


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(598, 450)
        self.icon = QIcon(":/icons/uglytheme/48x48/polibeepsync.png")
        Form.setWindowIcon(self.icon)
        Form.setLocale(QLocale(QLocale.English, QLocale.UnitedStates))
        self.verticalLayout = QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QTabWidget(Form)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QWidget()
        self.tab.setObjectName("tab")
        self.horizontalLayout = QHBoxLayout(self.tab)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QLabel(self.tab)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)
        self.label = QLabel(self.tab)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)
        self.password = QLineEdit(self.tab)
        self.password.setMaximumSize(QSize(139, 16777215))
        self.password.setEchoMode(QLineEdit.Password)
        self.password.setObjectName("password")
        self.gridLayout.addWidget(self.password, 3, 1, 1, 1)
        self.userCode = QLineEdit(self.tab)
        self.userCode.setMaximumSize(QSize(139, 16777215))
        self.userCode.setText("")
        self.userCode.setObjectName("userCode")
        self.gridLayout.addWidget(self.userCode, 1, 1, 1, 1)
        spacerItem = QSpacerItem(40, 20, QSizePolicy.Expanding,
                                 QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 2, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.trylogin = QPushButton(self.tab)
        self.trylogin.setMaximumSize(QSize(154, 16777215))
        self.trylogin.setObjectName("trylogin")
        self.verticalLayout_2.addWidget(self.trylogin)
        self.login_attempt = QLabel(self.tab)
        self.login_attempt.setText("Logging in, please wait.")
        self.login_attempt.setStyleSheet("color: rgba(0, 0, 0, 0);")
        self.login_attempt.setObjectName("login_attempt")
        self.verticalLayout_2.addWidget(self.login_attempt)
        spacerItem1 = QSpacerItem(20, 40, QSizePolicy.Minimum,
                                  QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_4 = QLabel(self.tab)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        self.rootfolder = QLineEdit(self.tab)
        self.rootfolder.setMinimumSize(QSize(335, 0))
        self.rootfolder.setMaximumSize(QSize(335, 16777215))
        self.rootfolder.setInputMask("")
        self.rootfolder.setReadOnly(True)
        self.rootfolder.setObjectName("rootfolder")
        self.horizontalLayout_3.addWidget(self.rootfolder)
        self.changeRootFolder = QPushButton(self.tab)
        self.changeRootFolder.setObjectName("changeRootFolder")
        self.horizontalLayout_3.addWidget(self.changeRootFolder)
        spacerItem2 = QSpacerItem(40, 20, QSizePolicy.Expanding,
                                  QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_5 = QLabel(self.tab)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.timerMinutes = QSpinBox(self.tab)
        self.timerMinutes.setObjectName("timerMinutes")
        self.horizontalLayout_5.addWidget(self.timerMinutes)
        self.label_6 = QLabel(self.tab)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_5.addWidget(self.label_6)
        self.syncNow = QPushButton(self.tab)
        self.syncNow.setObjectName("syncNow")
        self.horizontalLayout_5.addWidget(self.syncNow)
        spacerItem3 = QSpacerItem(40, 20, QSizePolicy.Expanding,
                                  QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.addSyncNewCourses = QCheckBox(self.tab)
        self.addSyncNewCourses.setObjectName("addSyncNewCourses")
        self.verticalLayout_2.addWidget(self.addSyncNewCourses)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QWidget()
        self.tab_2.setObjectName("tab_2")
        self.horizontalLayout_2 = QHBoxLayout(self.tab_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.refreshCourses = QPushButton(self.tab_2)
        self.refreshCourses.setObjectName("refreshCourses")
        self.horizontalLayout_6.addWidget(self.refreshCourses)
        spacerItem4 = QSpacerItem(40, 20, QSizePolicy.Expanding,
                                  QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem4)
        self.verticalLayout_3.addLayout(self.horizontalLayout_6)
        self.coursesView = CoursesListView(self.tab_2)
        self.coursesView.setObjectName("coursesView")
        self.verticalLayout_3.addWidget(self.coursesView)
        self.horizontalLayout_2.addLayout(self.verticalLayout_3)
        self.tabWidget.addTab(self.tab_2, "")
        self.tab_3 = QWidget()
        self.tab_3.setObjectName("tab_3")
        self.horizontalLayout_7 = QHBoxLayout(self.tab_3)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.about = QPushButton(self.tab_3)
        self.about.setObjectName("about")
        self.horizontalLayout_8.addWidget(self.about)
        spacerItem5 = QSpacerItem(40, 20, QSizePolicy.Expanding,
                                  QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem5)
        self.verticalLayout_4.addLayout(self.horizontalLayout_8)
        self.status = QTextEdit(self.tab_3)
        self.status.setTextInteractionFlags(
            Qt.TextSelectableByKeyboard | Qt.TextSelectableByMouse)
        self.status.setObjectName("status")
        self.verticalLayout_4.addWidget(self.status)
        self.horizontalLayout_7.addLayout(self.verticalLayout_4)
        self.tabWidget.addTab(self.tab_3, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.okButton = QDialogButtonBox(Form)
        self.okButton.setStandardButtons(QDialogButtonBox.Ok)
        self.okButton.setObjectName("okButton")
        self.okButton.clicked.connect(self.hide)
        self.verticalLayout.addWidget(self.okButton)

        self.retranslateUi(Form)
        self.tabWidget.setCurrentIndex(0)
        QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle("poliBeePsync")
        self.label_2.setText(QApplication.translate("Form", "Password", None,
                                                    QApplication.UnicodeUTF8))
        self.label.setText(QApplication.translate("Form", "User code", None,
                                                  QApplication.UnicodeUTF8))
        self.trylogin.setText(
            QApplication.translate("Form", "Try login credentials", None,
                                   QApplication.UnicodeUTF8))
        self.login_attempt.setText(
            QApplication.translate("Form", "Login successful", None,
                                   QApplication.UnicodeUTF8))
        self.label_4.setText(
            QApplication.translate("Form", "Root folder", None,
                                   QApplication.UnicodeUTF8))
        self.changeRootFolder.setText(
            QApplication.translate("Form", "Change", None,
                                   QApplication.UnicodeUTF8))
        self.label_5.setText(QApplication.translate("Form", "Sync every", None,
                                                    QApplication.UnicodeUTF8))
        self.label_6.setText(QApplication.translate("Form", "minutes", None,
                                                    QApplication.UnicodeUTF8))
        self.syncNow.setText(QApplication.translate("Form", "Sync now", None,
                                                    QApplication.UnicodeUTF8))
        self.addSyncNewCourses.setText(QApplication.translate("Form",
                                                              "Automatically add and sync new available courses",
                                                              None,
                                                              QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab),
                                  QApplication.translate("Form",
                                                         "General settings",
                                                         None,
                                                         QApplication.UnicodeUTF8))
        self.refreshCourses.setText(
            QApplication.translate("Form", "Refresh list", None,
                                   QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2),
                                  QApplication.translate("Form", "Courses",
                                                         None,
                                                         QApplication.UnicodeUTF8))
        self.about.setText(QApplication.translate("Form", "About", None,
                                                  QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3),
                                  QApplication.translate("Form", "Status",
                                                         None,
                                                         QApplication.UnicodeUTF8))


from polibeepsync import icone_rc



from os import environ
from polibeepsync.common import User
from polibeepsync.common import InvalidLoginError
import requests
from behave import *

@given('I have correct username and password')
def step_impl(context):
    usercode = environ["USERCODE"]
    password = environ["PASSWORD"]
    user = User(usercode, password)
    context.user = user

@given('the website is reachable')
def step_impl(context):
    try:
        context.user.visit()
    except (requests.ConnectionError, requests.Timeout):
        assert False
    else:
        assert True

@when('I visit the login page')
def step_impl(context):
    context.user.visit()

@then("I should access the private area")
def step_impl(context):
    try:
        context.user.login()
    except IndexError:
        assert context.user.logged is True

@given('I have wrong username or password')
def step_impl(context):
    usercode = 99999999
    password = 'abcdef'
    user = User(usercode, password)
    context.user= user

@then('I should get an exception "Invalid username or password"')
def step_impl(context):
    try:
        context.user.login()
    except InvalidLoginError:
        assert True

@then('logged variable is false')
def step_impl(context):
    context.user.logged is False


from behave import *
from polibeepsync.common import User, Course, Courses, CourseFile

@when('I ask for the courses')
def step_impl(context):
    guy= User('fakeid', 'fakepwd')
    course_a = Course('a', 'alink')
    course_b = Course('b', 'blink')
    guy.available_courses.append(course_a, course_b)
    context.courses = guy.available_courses


@then('I get the courses list')
def step_impl(context):
    # not a great test
    assert len(context.courses) > 0

@when('I select a course')
def step_impl(context):
    fake_course = Course('fakename', 'fakelink')
    file_a = CourseFile('filea', '03/11/14 12.33')
    file_b = CourseFile('fileb', '06/09/14 11.25')
    fake_course.append(file_a, file_b)
    context.selected_course = fake_course

@when('I ask for files')
def step_impl(context):
    context.files = context.selected_course.files
    # chiamare metodo che riceve i file per un corso
    #context.files = context.selected_course.files

@then('I get files')
def step_impl(context):
    # not a great test
    assert len(context.files) == 2

@when('a new course is available')
def step_impl(context):
    new_course = Course('newcourse', 'fakelink')
    common_course = Course('common', 'commonlink')
    guy = User('fakeid', 'fakepwd')
    guy.available_courses.append(common_course)
    context.guy = guy
    online_courses = Courses()
    online_courses.append(new_course, common_course)
    context.online_courses = online_courses

@then('it should be added to the local copy')
def step_impl(context):
    context.guy.sync_available_courses(context.online_courses)
    assert context.guy.available_courses == context.online_courses

@when('a course is no more available')
def step_impl(context):
    new_course = Course('newcourse', 'fakelink')
    common_course = Course('common', 'commonlink')
    guy = User('fakeid', 'fakepwd')
    guy.available_courses.append(new_course, common_course)
    context.guy = guy
    online_courses = Courses()
    online_courses.append(common_course)
    context.online_courses = online_courses

@then('it should be removed from the local copy')
def step_impl(context):
    context.guy.sync_available_courses(context.online_courses)
    assert context.guy.available_courses == context.online_courses


import pytest
import os
from polibeepsync.filesettings import settingsFromFile, settingsToFile
from configparser import RawConfigParser


@pytest.fixture
def default_settings_dict():
    conf = {
        'UpdateEvery': '60',
        'RootFolder': 'root',
        'SyncNewCourses': 'False'
    }
    return conf


@pytest.fixture
def default_settings_file(tmpdir):
    conf = RawConfigParser()
    conf.optionxform = lambda option: option
    conf['General'] = {
        'UpdateEvery': '60',
        'RootFolder': 'root',
        'SyncNewCourses': 'False'
    }
    f = tmpdir.mkdir('fold').join('sets.ini')
    with open(str(f), 'w') as setsfile:
        conf.write(setsfile)
    return str(f)


def test_allvaluesvalidpresent(default_settings_file, default_settings_dict):
    """All values are present in the config file
    All values are valid
    File is accessible"""
    sets = settingsFromFile(default_settings_file, default_settings_dict)
    assert sets == default_settings_dict


def test_notallvaluesarepresent(tmpdir):
    """Not all values are present; all are valid, file is accessible"""
    conf = RawConfigParser()
    conf.optionxform = lambda option: option
    conf['General'] = {
        'UpdateEvery': '50',
        'RootFolder': 'custom',
    }
    f = tmpdir.mkdir('fold').join('sets.ini')
    with open(str(f), 'w') as setsfile:
        conf.write(setsfile)
    sets = settingsFromFile(str(f), default_settings_dict())
    merge = {
        'UpdateEvery': '50',
        'RootFolder': 'custom',
        'SyncNewCourses': 'False'
    }
    assert sets == merge


def test_allpresent_someinvalid(tmpdir):
    """File accessible, all values present, some not valid"""
    conf = RawConfigParser()
    conf.optionxform = lambda option: option
    conf['General'] = {
        'UpdateEvery': '-10',
        'SyncNewCourses': 'randomtext'
    }
    f = tmpdir.mkdir('fold').join('sets.ini')
    with open(str(f), 'w') as setsfile:
        conf.write(setsfile)
    sets = settingsFromFile(str(f), default_settings_dict())
    nice = {
        'UpdateEvery': '60',
        'RootFolder': 'root',
        'SyncNewCourses': 'False'
    }
    assert sets == nice


def test_somepresent_allinvalid(tmpdir):
    """Not all values present, all are invalid, file accessible"""
    conf = RawConfigParser()
    conf.optionxform = lambda option: option
    conf['General'] = {
        'UpdateEvery': 'noatanumer',
        'SyncNewCourses': 'False'
    }
    f = tmpdir.mkdir('fold').join('sets.ini')
    with open(str(f), 'w') as setsfile:
        conf.write(setsfile)
    sets = settingsFromFile(str(f), default_settings_dict())
    nice = {
        'UpdateEvery': '60',
        'RootFolder': 'root',
        'SyncNewCourses': 'False'
    }
    assert sets == nice


def test_missingsection(tmpdir):
    f = tmpdir.mkdir('fold').join('sets.ini')
    text = """UpdateEvery = 60\nRootFOlder = root\n
\nSyncNewCourses = True"""
    with open(str(f), 'w') as setsfile:
        setsfile.write(text)
    sets = settingsFromFile(str(f), default_settings_dict())
    assert sets == default_settings_dict()


def test_missingfolder():
    sets = settingsFromFile('/i/dont/exist/sets.ini', default_settings_dict())
    assert sets == default_settings_dict()


def test_missingfile(tmpdir):
    f = tmpdir.mkdir('fold')
    sets = settingsFromFile(str(f)+"nofile.ini", default_settings_dict())
    assert sets == default_settings_dict()


def test_savefileexists(tmpdir, default_settings_dict):
    f = tmpdir.mkdir('fold').join('sets.ini')
    conf = RawConfigParser()
    conf.optionxform = lambda option: option
    conf['General'] = {
        'UpdateEvery': '30',
        'RootFolder': 'anotherroot',
        'SyncNewCourses': 'False'
    }
    with open(str(f), 'w') as setsfile:
        conf.write(setsfile)
    newsettings = {
        'UpdateEvery': '20',
        'RootFolder': 'finalroot',
        'SyncNewCourses': 'True'
    }
    settingsToFile(newsettings, str(f))
    assert settingsFromFile(str(f), default_settings_dict) == newsettings


def test_saveioerror(tmpdir):
    """a generic IOError, for example we don't have write permission"""
    f = tmpdir.mkdir('fold').join('sets.ini')
    f.write('doesntmatter')
    os.chmod(str(f), 0o444)
    try:
        newsettings = {
            'UpdateEvery': '20',
            'RootFolder': 'finalroot',
            'SyncNewCourses': 'True'
        }
        settingsToFile(newsettings, str(f))
    except IOError:
        pass



from setuptools import setup
import os
import re


here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

def read(*names, **kwargs):
    with open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")
    ) as fp:
        return fp.read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


classifiers = [
    "Development Status :: 4 - Beta",
    "Environment :: X11 Applications :: Qt",
    "Intended Audience :: Education",
    "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3.4"
]

setup(
    name='poliBeePsync',
    version = find_version("polibeepsync/__init__.py"),
    url = "http://www.davideolianas.com/polibeepsync",
    author= "Davide Olianas",
    author_email= "ubuntupk@gmail.com",
    license = 'GPLv3+',
    install_requires = [
        "requests",
        "appdirs",
        "PySide",
        "beautifulsoup4"
    ],
    packages = ['polibeepsync'],
    description = "Sync files from https://beep.metid.polimi.it "
                 "(for students of Politecnico di Milano)",
    long_description = long_description,
    classifiers = classifiers,
    entry_points = {
    'console_scripts': [
        'polibeepsync=polibeepsync.qtgui:main',
    ],
    'gui_scripts': [
        'polibeepsync-gui=polibeepsync.qtgui:main'
    ]
}
)


__copyright__ = "Copyright 2014 Davide Olianas (ubuntupk@gmail.com)."

__license__ = """This file is part of poliBeePsync.
poliBeePsync is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

poliBeePsync is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with poliBeePsync. If not, see <http://www.gnu.org/licenses/>.
"""

from bs4 import BeautifulSoup
from datetime import datetime, timedelta, tzinfo
import requests
import os
import logging


class InvalidLoginError(Exception):
    pass


class CourseNotFoundError(Exception):
    pass


class GMT1(tzinfo):
    def utcoffset(self, dt):
        return timedelta(hours=1) + self.dst(dt)

    def dst(self, dt):
        # DST starts last Sunday in March
        d = datetime(dt.year, 4, 1)   # ends last Sunday in October
        self.dston = d - timedelta(days=d.weekday() + 1)
        d = datetime(dt.year, 11, 1)
        self.dstoff = d - timedelta(days=d.weekday() + 1)
        if self.dston <= dt.replace(tzinfo=None) < self.dstoff:
            return timedelta(hours=1)
        else:
            return timedelta(0)

    def tzname(self, dt):
        return "GMT +1"


class GenericSet(object):
    def __init__(self):
        self.elements = []

    def _elements_names(self):
        return [elem.name for elem in self.elements]

    def __eq__(self, other):
        this_elements = set(self.elements)
        other_elements = set(other.elements)
        if this_elements == other_elements:
            return True
        else:
            return False

    def __contains__(self, key):
        if key.name in self._elements_names():
            return True
        else:
            return False

    def __getitem__(self, key):
        if key in self._elements_names():
            for elem in self.elements:
                if elem.name == key:
                    return elem
        else:
            raise KeyError

    def __iter__(self):
        return iter(self.elements)

    def __sub__(self, other):
        return list(set(self.elements) - set(other.elements))

    def append(self, *args):
        for elem in args:
            if not self.__contains__(elem):
                self.elements.append(elem)

    def remove(self, arg):
        if arg in self.elements:
            self.elements.remove(arg)

    def __len__(self):
        return len(self.elements)


class Courses(GenericSet):
    def __hash__(self):
        unordered_name = self._elements_names()
        return hash("".join(sorted(unordered_name)))

    def __repr__(self):
        before = "Courses collection:\n"
        texts = [elem.name for elem in self.elements]
        joined_texts = "\n".join(texts)
        return before + joined_texts


class Course(GenericSet):
    def __init__(self, name, documents_url, sync=False):
        logging.debug('Creating course with name={},'
                      ' documents_url={}, sync={}'
                      .format(name, documents_url, sync))
        super(Course, self).__init__()
        self.name = name
        self.documents_url = documents_url
        self.sync = sync
        self.documents = ""
        self.save_folder_name = ""

    def simplify_name(self, name):
        upper = name
        try:
            rmleftbrackets = "".join(name.split('[')[1:-1])
            upper = rmleftbrackets.split("]")[1].lstrip(' - ').rstrip(" ")
            #upper = name.split('[')[1].split(']')[1].rstrip(" ").lstrip(' - ')
        except Exception as err:
            logging.error('Failed to simplify course name {}'.format(name))
            logging.error(str(err))
        return upper.title()

    def __hash__(self):
        return hash(self.name)

    def __repr__(self):
        return 'Course {}'.format(self.name)

    def __contains__(self, key):
        if key.name in self._elements_names():
            return True
        else:
            return False

    def __eq__(self, other):
        if self.name == other.name:
            return True
        else:
            return False


class CourseFile(object):
    def __init__(self, name, url, last_online_edit_time):
        gmt1 = GMT1()
        self.name = name
        self.url = url
        self.last_online_edit_time = last_online_edit_time
        self.local_creation_time = datetime(1990, 1, 1, 1, 1,
                                     tzinfo=gmt1)

    def __hash__(self):
        return hash(self.name)


class Folder(object):
    def __init__(self, name, url):
        self.name = name
        self.url = url
        self.files = []
        self.folders = []


class User(object):
    loginurl = 'https://beep.metid.polimi.it/polimi/login'
    gmt1 = GMT1()

    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.session = requests.Session()
        self.logged = False
        self.subscribed_courses = []
        self.courses_url = ""
        self.available_courses = Courses()
        self.root_save_folder = ""


    def logout(self):
        """Logout.

        It re-creates a session and sets :attr:`logged` to ``False``."""
        del(self.session)
        self.session = requests.Session()
        #self.session.cookies.clear()
        self.logged = False

    def get_page(self, url):
        """Use this method to get a webpage.

        It will check if the session is expired, and relogin if necessary.

        Returns:
            response (:class:`requests.Response`): a :class:`requests.Response`
            instance
        """
        response = self.session.get(url, timeout=5, verify=True)
        soup = BeautifulSoup(response.text)
        login_tag = soup.find('input', attrs={'id': 'login'})
        if login_tag is not None:
            logging.info("The session has expired. Logging-in again...")
            self.logout()
            self.login()
            response = self.session.get(url, timeout=5, verify=True)
        return response

    def get_file(self, url):
        """Use this method to get a file.

        It will check if the session is expired, and re-login if necessary.
        The file bytes can be accessed with the :attr:`content` attribute

        >>> user = User('username', 'password')
        >>> response = user.get_file('url_to_file', timeout=5, verify=True)
        >>> with open('outfile','wb') as f:
        ...    f.write(response.content)

        Returns:
            response (requests.Response): a :class:`requests.Response` object
        """
        response = self.session.get(url, timeout=5, verify=True)
        if len(response.history) > 0:
            # it means that we've been redirected to the login page
            logging.info("The session has expired. Logging-in again...")
            self.logout()
            self.login()
            response = self.session.get(url, timeout=5, verify=True)
        return response

    def login(self):
        """Try logging in.

        If the login is successful, :attr:`logged` is set to ``True``.
        If it fails, :attr:`logged` is set to ``False`` and raises an
        :class:`InvalidLoginError`.

        Raises:
            InvalidLoginError: when the login fails
        """
        # switch to english version if we're on the italian site
        default_lang_page = self.session.get(self.loginurl, timeout=5, verify=True)
        lang_soup = BeautifulSoup(default_lang_page.text)
        lang_tag = lang_soup.find('a', attrs={'title': 'English'})
        if lang_tag:
            self.session.get('https://aunicalogin.polimi.it' +
                             lang_tag['href'], timeout=5, verify=True)
        payload = "login=%s&password=%s" % (self.username, self.password) + \
                  '&evn_conferma%3Devento=Accedi'
        login_headers = {
            'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:34.0)\
Gecko/20100101 Firefox/34.0',

            'Content-Type': 'application/x-www-form-urlencoded',
            'Content-Length': len(payload),
            'Accept': 'text/html,application/xhtml+xml,\
            application/xml;q=0.9,*/*;q=0.8'
        }
        login_response = self.session.post(
            'https://aunicalogin.polimi.it:443/aunicalogin/\
aunicalogin/controller/IdentificazioneUnica.do?\
&jaf_currentWFID=main',
            data=payload, headers=login_headers)
        login_soup = BeautifulSoup(login_response.text)
        try:
            parenttag = login_soup.find_all('table')[3]
            parenttag.find('td',
                           text='\n\t\t\t\t\t\n\t\t\t\t\t\t\n\t\t\t\t\
\t\tCode: 14 - Identificazione fallita\n\t\t\t\t\t\n\t\t\t\t')
        except IndexError:
            hidden_fields = BeautifulSoup(login_response.text).find_all(
                'input', attrs={'type': 'hidden'}
            )
            # The SAML response wants '+' replaced by %2B
            relay_state = hidden_fields[0].attrs['value']
            saml_response = hidden_fields[1].attrs['value'].replace('+',
                                                                    '%2B')
            final_request_data = "RelayState=%s&SAMLResponse=%s" % \
                                 (relay_state, saml_response)
            final_headers = {
                'Cookie': "GUEST_LANGUAGE_ID=en_GB; \
COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER",
                'Content-Type': 'application/x-www-form-urlencoded',
                'Content-Length': len(final_request_data),
            }
            self.session.post(
                'https://beep.metid.polimi.it/Shibboleth.sso/SAML2/POST',
                data=final_request_data,
                headers=final_headers)
            cookies = requests.utils.dict_from_cookiejar(self.session.cookies)
            for key in cookies:
                if key.startswith('_shibsession'):
                    shibsessionstr = "%s=%s" % (key, cookies[key])
            main_headers = {
                'Cookie': "GUEST_LANGUAGE_ID=en_GB; \
COOKIE_SUPPORT=true; polij_device_category=PERSONAL_COMPUTER; %s" % (
                    shibsessionstr)
            }
            mainpage = self.session.get(
                'https://beep.metid.polimi.it/polimi/login',
                headers=main_headers, timeout=5, verify=True)
            self.courses_url = mainpage.url
            self.logged = True
        else:
            self.logged = False
            raise InvalidLoginError

    def get_online_courses(self):
        """Return the courses available online.

        Returns:
            online_courses (:class:`Courses`): a :class:`Courses` container of
            all courses available online."""
        coursespage = self.get_page(self.courses_url)
        courses_soup = BeautifulSoup(coursespage.text)
        raw_courses = courses_soup.find_all('tr',
                                            attrs={'class': 'results-row'})
        # the first tag is not a course
        raw_courses.pop(0)
        online_courses = Courses()
        # we iterate over the tags
        for course in raw_courses:
            firstlink = course.td.a['href']
            name = course.td.a.strong.text
            # the real link is found after a redirect
            link = self.get_page(firstlink).url
            link = link.rstrip('attivita-online-e-avvisi')
            weird_parameters = ['_20_folderId=0',
                                '_20_displayStyle=list',
                                '_20_viewEntries=0',
                                '_20_viewFolders=0',
                                '_20_entryEnd=500',
                                '_20_entryStart=0',
                                '_20_folderEnd=500',
                                '_20_folderStart=0',
                                '_20_viewEntriesPage=1',
                                'p_p_id=20',
                                'p_p_lifecycle=0'
                                ]
            link = link + 'documenti-e-media?' + "&".join(weird_parameters)
            # we ignore BeeP channel
            if str(name) != "BeeP channel":
                course = Course(name, link)
                online_courses.append(course)
        return online_courses

    def sync_available_courses(self, master_courses):
        """Sync :attr:`available_courses` to :attr:`master_courses`.

        This function will compare the courses in master_courses; if some of
        them are not present in the available_courses instance attribute, they
        will be added to available_courses; if any course is present in
        self.available_courses but not in master_courses, it will be
        removed from self.available_courses.

        A typical usage would be the following

        >>> user = User('fakeid', 'fakepwd')
        >>> user.login()
        >>> online = user.get_online_courses()
        >>> user.sync_available_courses(online)

        Args:
            master_courses (Courses): The updated :class:`Courses` instance
        """
        # New courses should be added.
        # the "if not in" check is not really needed, since the append
        # function doesn't allow duplicates.
        for elem in master_courses:
            if elem not in self.available_courses:
                self.available_courses.append(elem)
        # now do the opposite: if a course has been deleted,
        # do the same with the local copy
        for elem in self.available_courses:
            if elem not in master_courses:
                self.available_courses.remove(elem)

    def update_course_files(self, course):
        rootfolder = self.find_files_and_folders(course.documents_url,
                                                 'rootfolder')
        course.documents = rootfolder

    def find_files_and_folders(self, link, thisfoldername):
        response = self.get_page(link)
        soup = BeautifulSoup(response.text)
        tags = soup.find_all('span', attrs={'class': 'taglib-text'})
        tags = [elem for elem in tags if elem.text != ""]

        tags.pop(0)
        tags.pop(0)

        rawdates = [elem.parent.parent.parent.next_sibling.next_sibling.
                    next_sibling.next_sibling.next_sibling.next_sibling
                    for elem in tags]
        last_column = [elem.next_sibling.next_sibling.next_sibling.
                       next_sibling for elem in rawdates]

        folder = Folder(thisfoldername, response.url)

        for i, v in enumerate(tags):
            name = v.text
            rawdate = rawdates[i]
            day = int(rawdate.text.split(' ')[1].split('/')[0])
            month = int(rawdate.text.split(' ')[1].split('/')[1])
            year = int(rawdate.text.split(' ')[1].split('/')[2])
            hour = int(rawdate.text.split(' ')[2].split('.')[0])
            minute = int(rawdate.text.split(' ')[2].split('.')[1])
            complete_date = datetime(year, month, day, hour, minute,
                                     tzinfo=self.gmt1)
            download_link = last_column[i].find_all('a')
            elem = download_link[2]
            if elem.text.startswith('  Download ('):
                link = elem['href']
                complete_file = CourseFile(name, link, complete_date)
                folder.files.append(complete_file)

            else:
                link = v.parent['href']
                subfolder = self.find_files_and_folders(link, name)
                folder.folders.append(subfolder)
        return folder


    def save_files(self, masterfolder, out_rootfolder):
        for coursefile in masterfolder.files:
            fname = os.path.join(out_rootfolder, coursefile.name)
            basenames = [os.path.splitext(os.path.basename(f))[0]
                         for f in os.listdir(out_rootfolder)
                         if os.path.isfile(os.path.join(out_rootfolder, f))]

            if os.path.exists(fname) and\
                coursefile.local_creation_time < \
                coursefile.last_online_edit_time:
                result = self.get_file(coursefile.url)
                complete_basename = result.headers['Content-Disposition']\
                .split("; ")[1].split("=")[1].strip('"')
                complete_name = os.path.join(out_rootfolder, complete_basename)
                with open(complete_name, 'wb') as f:
                    f.write(result.content)
                coursefile.local_creation_time = datetime.now(self.gmt1)
            elif fname in basenames and\
                coursefile.local_creation_time < \
                coursefile.last_online_edit_time:
                result = self.get_file(coursefile.url)
                complete_basename = result.headers['Content-Disposition']\
                .split("; ")[1].split("=")[1].strip('"')
                complete_name = os.path.join(out_rootfolder, complete_basename)
                with open(complete_name, 'wb') as f:
                    f.write(result.content)
                coursefile.local_creation_time = datetime.now(self.gmt1)
            elif not (os.path.exists(fname) and fname in basenames):
                result = self.get_file(coursefile.url)
                complete_basename = result.headers['Content-Disposition']\
                .split("; ")[1].split("=")[1].strip('"')
                complete_name = os.path.join(out_rootfolder, complete_basename)
                with open(complete_name, 'wb') as f:
                    f.write(result.content)
                coursefile.local_creation_time = datetime.now(self.gmt1)
        if masterfolder.name == "rootfolder":
            for folder in masterfolder.folders:
                path = os.path.join(out_rootfolder, folder.name)
                os.makedirs(path, exist_ok=True)
                self.save_files(folder, path)
        else:
            for folder in masterfolder.folders:
                path = os.path.join(out_rootfolder, folder.name)
                os.makedirs(path, exist_ok=True)
                self.save_files(folder, path)


from behave import *

@when('a file is not available locally')
def step_impl(context):
    assert False

@then('I should get a local copy')
def step_impl(context):
    assert False

@when('a file on BeeP is newer than the local copy')
def step_impl(context):
    assert False

@then('I should get the updated file')
def step_impl(context):
    assert False

@when('a local file doesn\'t exist on BeeP')
def step_impl(context):
    assert False

@then('nothing should happen')
def step_impl(context):
    assert False